/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"
#include <gpmm.h>

cvar_t bgmvolume = {"bgmvolume", "1", true};
cvar_t volume = {"volume", "0.7", true};

unsigned short RingBuf[32768];
 
void S_Init (void)
{
  GpPcmInit(PCM_M11,PCM_8BIT);
  memset(RingBuf,0,32768);
  GpPcmPlay(RingBuf,32768,1);
}

void S_AmbientOff (void)
{
}

void S_AmbientOn (void)
{
}

void S_Shutdown (void)
{
  GpPcmStop(); 
}

void S_TouchSound (char *sample)
{
}

void S_ClearBuffer (void)
{
}

void S_StaticSound (sfx_t *sfx, vec3_t origin, float vol, float attenuation)
{
  int size,h;
  unsigned short *sample;

  size = COM_OpenFile (va("sound/%s",sfx->name), &h);
  COM_CloseFile(h);

  sample=(unsigned short *)COM_LoadTempFile(va("sound/%s",sfx->name));
  if (sample==NULL) {
    GpError(va("sound/%s load failed",sfx->name),69);
  }
  if (size<=32768) {
    memcpy(RingBuf,sample,size);
    memcpy(RingBuf+size,0,(32768)-size);
  }
  //  GpPcmPlay(sample,size*0.9,0);
}

void S_StartSound (int entnum, int entchannel, sfx_t *sfx, vec3_t origin, float fvol,  float attenuation)
{
  
  int size,h;
  unsigned short *sample;

  size = COM_OpenFile (va("sound/%s",sfx->name), &h);
  COM_CloseFile(h);

  sample=(unsigned short *)COM_LoadTempFile(va("sound/%s",sfx->name));
  if (sample==NULL) {
    GpError(va("sound/%s load failed",sfx->name),69);
  }

  if (size<=32768) {
    memcpy(RingBuf,sample,size);
    memcpy(RingBuf+size,0,(32768)-size);
  }
  //  GpPcmPlay(sample,size*0.9,0);
  
}

void S_StopSound (int entnum, int entchannel)
{
}

sfx_t *S_PrecacheSound (char *sample)
{
  
	sfx_t	*sfx;
	sfx=gm_malloc(sizeof(sfx_t));

	strcpy(sfx->name,sample);
	return sfx;

}

void S_ClearPrecache (void)
{
}

void S_Update (vec3_t origin, vec3_t v_forward, vec3_t v_right, vec3_t v_up)
{	
}

void S_StopAllSounds (qboolean clear)
{
}

void S_BeginPrecaching (void)
{
}

void S_EndPrecaching (void)
{
}

void S_ExtraUpdate (void)
{
}

void S_LocalSound (char *s)
{
  int size,h;
  unsigned short *sample;

  size = COM_OpenFile (va("sound/%s",s), &h);
  COM_CloseFile(h);

  sample=(unsigned short *)COM_LoadTempFile(va("sound/%s",s));
  if (sample==NULL) {
    GpError(va("sound/%s load failed",s),69);
  }

  if (size<=32768) {
    memcpy(RingBuf,sample,size);
    memcpy(RingBuf+size,0,(32768)-size);
  }
  //  GpPcmPlay(sample,size*0.9,0);
}

