/************************************************************************
 *									*
 *            DIR.C							*
 *            =====							*
 *                                        W.G. Reissnegger		*
 *                                        C&C Research Labs		*
 *                                        NEC USA Inc.			*
 *                                        4 Independence Way		*
 *                                        Princeton NJ 08540		*
 *									*
 *   DOS FAT12 and FAT16 directory operations                           *
 *                                                                      *
 *   Copyright (C) 1997, 1998, 1999, 2000 NEC USA, Inc.			*
 *                 All Rights Reserved					*
 *									*
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License as     *
 *   published by the Free Software Foundation; either version 2 of     *
 *   the license, or (at your option) any later version.                *
 *									*
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public          *
 *   License along with this program;  if not, write to the             *
 *       Free Software Foundation, Inc.,                                *
 *       59 Temple Place - Suite 330,                                   *
 *       Boston, MA 02111-1307                                          *
 *                                                                      *
 ************************************************************************/

#include <rome.h>
#include <rome_if.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <nvram.h>
#include "msdos.h"

#define DIR_MEM_DEBUG			0xd1
#define DIR_MAX_PATHLEN			256

/************************************************************************ 
 *									*
 *            Internal Routines						*
 *            =================						*
 *									*
 ************************************************************************/

static void dir_split_path(char *s, char **base, char **entry)
{
    char *p = s + strlen(s) - 1;

    /* skip leading '/' */

    while ('/' == *s)
    {
	s++;
    }

    /* skip terminating '/' */

    while ('/' == *p)
    {
	*p-- = '\0';
    }

    while ((p != s) && ('/' != *p))
    {
	p--;
    }

    if ('/' == *p)
    {
	*p = '\0';
	*entry = ++p;
	*base = s;
    }
    else
    {
	*entry = s;
	*base = "";
    }
}

static int dir_make83(char *name83, const char *filename)
{
    char *p;
    int slen;
    int plen;

    memset(name83, ' ', 11);

    p = strchr(filename, '.');
    if (p)
    {
	plen = p - filename;
	slen = strlen(p + 1);
    }
    else
    {
	plen = strlen(filename);
	slen = 0;
    }

    if ((plen > 8) || (slen > 3))
    {
	return -1;
    }

    if (plen)
    {
	while (plen--)
	{
	    name83[plen] = (char) toupper(filename[plen]);
	}
    }
    if (slen)
    {
	while (slen--)
	{
	    name83[slen + 8] = (char) toupper(p[slen + 1]);
	}
    }

    return 0;
}

static int dir_find_free_entry_in_cluster(FILE *fp,
					  MSDOS_FS_INFO *inf,
					  ushort start_cluster,
					  MSDOS_DIR_ENTRY **entry,
					  ROME_MESSAGE *msg)
{
    MSDOS_DIR_ENTRY *e;
    int i;
    int nr_entry = inf->blksize / sizeof(MSDOS_DIR_ENTRY);
    ushort cluster = start_cluster;

    do
    {
	if  (start_cluster)
	{
	    /* read as subdir */

	    dos_fs_read_cluster(fp, inf, start_cluster, msg);
	}
	else
	{
	    /* read as rootdir cluster */

	    dos_fs_read_dir_cluster(fp, inf, cluster, msg);
	}

	e = (MSDOS_DIR_ENTRY *)msg->b_rptr;

	for (i = 0; nr_entry > i; i++)
	{
	    /* deleted and non-attr files skip */

	    if ((DELETED_FLAG == (uchar)e->name[0]) ||
		(0 == e->name[0]) ||	/* end of dir */
		(ATTR_NONE == e->attr))
	    {
		*entry = e;
		/* if we found the entry, we don't return the mblk.
		 * The caller is responsible for returning it!!!
		 */
		return 0;
	    }
	    e++;
	}
	rome_retmblk(msg);

	/* this increment will only have effect, if we are in the
	 * root directory and (0 == start_cluster)
	 */
	cluster++;
    }
    while (0 == start_cluster);

    rome_kprintf("dir_find_free_entry_in_cluster(): FREE NOT FOUND\n");
    return -1;
}

static int dir_find_entry_in_cluster(FILE *fp, MSDOS_FS_INFO *inf,
				     ushort start_cluster,
				     const char *name83,
				     MSDOS_DIR_ENTRY **entry,
				     ROME_MESSAGE *msg)
{
    MSDOS_DIR_ENTRY *e;
    int i;
    int nr_entry = inf->blksize / sizeof(MSDOS_DIR_ENTRY);
    ushort curr_cluster = start_cluster;

    do
    {
	if (start_cluster)
	{
	    /* read as subdir */

	    dos_fs_read_cluster(fp, inf, start_cluster, msg);
	}
	else
	{
	    /* read as rootdir cluster */

	    dos_fs_read_dir_cluster(fp, inf, curr_cluster, msg);
	}

	e = (MSDOS_DIR_ENTRY *)msg->b_rptr;

	for (i = 0; nr_entry > i; i++)
	{
	    /* end of dir test */

	    if  (0 == e->name[0])
	    {
		rome_retmblk(msg);
		return -1;
	    }

	    /* deleted and non-attr files skip */

	    if ((DELETED_FLAG == (uchar) e->name[0]) || (ATTR_NONE == e->attr))
	    {
		e++;
		continue;
	    }

	    /* name test */

	    if (0 == strncmp(e->name, name83, 11))
	    {
		*entry = e;
		/* if we found the entry, we don't return the mblk.
		 * The caller is responsible for returning it!!!
		 */
		return 0;
	    }
	    e++;
	}
	rome_retmblk(msg);

	/* this increment will only have effect, if we are in the
	 * root directory and (0 == start_cluster)
	 */
	curr_cluster++;
    }
    while (0 == start_cluster);

    rome_kprintf("dir_find_entry_in_cluster(): NOT FOUND\n");
    return -1;
}

static int dir_find_free_entry(FILE *fp,
			       MSDOS_FS_INFO *inf,
			       ushort start_cluster,
			       MSDOS_DIR_ENTRY **entry,
			       ROME_MESSAGE *msg)
{
    ROME_MESSAGE dir_msg;
    MSDOS_DIR_ENTRY *dir_entry;
    ushort curr_cluster = start_cluster;
    ushort new_fat_entry;
    MSDOS_DIR_ENTRY *e;

    do
    {
	if (0 == dir_find_free_entry_in_cluster(fp, inf, curr_cluster,
						&e, msg))
	{
	    *entry = e;
	    return 0;
	}

	if (curr_cluster)	/* if not root dir, get next dir cluster nr */
	{
	    curr_cluster = dos_fat_lookup_entry(curr_cluster, inf);
	    if (1 == curr_cluster)	/* EOF */
	    {
		break;
	    }
	}
    }
    while (start_cluster);	/* only continue if not root dir */

    if (!start_cluster)
    {
	rome_kprintf("dir_find_free_entry(): root directory full\n");
	return ENOSPC;
    }

    /* as we got here, there's no entry free, we have to get a new 
     * cluster for this subdir
     */

    if (0 == (new_fat_entry = dos_fat_lookup_free(inf)))
    {
	rome_kprintf("dir_find_free_entry(): Could not get FAT entry\n");
	return ENOSPC;
    }

    if (0 != dos_fat_write_entry(inf, new_fat_entry, FAT_EOF))
    {
	rome_kprintf("dir_find_free_entry(): Could not write FAT entry\n");
	return EIO;
    }

    /* write FAT chain link for new cluster */

    if (0 != dos_fat_write_entry(inf, curr_cluster, new_fat_entry))
    {
	rome_kprintf("dir_find_free_entry(): Could not write FAT link\n");
	return EIO;
    }

    /* clear the cluster for the new subdir */

    dos_fs_read_cluster(fp, inf, new_fat_entry, &dir_msg);
    dir_entry = (MSDOS_DIR_ENTRY *)dir_msg.b_rptr;
    dir_entry->name[0] = 0;	/* END OF DIR flag */
    dos_fs_write_cluster(fp, inf, new_fat_entry, &dir_msg);

    /* Now it actually should work, let's try it again */
    return dir_find_free_entry(fp, inf, start_cluster, entry, msg);
}

static int dir_find_entry(FILE * fp, MSDOS_FS_INFO *inf,
			  ushort start_cluster,
			  const char *name,
			  MSDOS_DIR_ENTRY **entry,
			  ROME_MESSAGE * msg)
{
    char name83[11];
    ushort curr_cluster = start_cluster;
    MSDOS_DIR_ENTRY *e;

    if (0 != dir_make83(name83, name))
    {
	return EINVAL;
    }

    do
    {
	if (0 == dir_find_entry_in_cluster(fp, inf, curr_cluster,
					   name83, &e, msg))
	{
	    *entry = e;
	    return 0;
	}

	if (curr_cluster)	/* if not root dir, get next dir cluster nr */
	{
	    curr_cluster = dos_fat_lookup_entry(curr_cluster, inf);
	    if (1 == curr_cluster)	/* EOF */
	    {
		break;
	    }
	}
    }
    while (start_cluster);	/* only continue if not root dir */

    return ENOENT;
}

static int dir_find_path(FILE *fp, MSDOS_FS_INFO *inf,
			 const char *full_path,
			 MSDOS_DIR_ENTRY **entry, ROME_MESSAGE *msg)
{
    int rc;
    MSDOS_DIR_ENTRY *e = NULL;
    char path[DIR_MAX_PATHLEN];
    char *p = path;
    char *curr;
    ushort cluster = 0;		/* start with root dir */
    int end_search = FALSE;

    if  (sizeof(path) <= strlen(full_path))
    {
	rome_kprintf("dir_find_path(): Path too long (%s)\n", full_path);
	return ENAMETOOLONG;
    }
    strcpy(path, full_path);

    /* lookup step by step */
    curr = p;
    do
    {
	while (*p && ('/' != *p))
	{
	    p++;
	}

	if (!*p)
	{
	    end_search = TRUE;
	}
	else
	{
	    *p = '\0';
	}

	/* check for the name in current dir */
	if (0 != (rc = dir_find_entry(fp, inf, cluster, curr, &e, msg)))
	{
	    return rc;
	}

	if (!end_search)
	{
	    cluster = e->start;

	    /* return the mblk we got */
	    rome_retmblk(msg);

	    curr = ++p;
	}
    }
    while (!end_search);

    if (!e)
    {
	return ENOENT;
    }

    *entry = e;
    return 0;
}

static int dir_create_entry(FILE *fp,
			    MSDOS_FS_INFO *inf,
			    const char *full_path,
			    MSDOS_DIR_ENTRY **e,
			    ROME_MESSAGE *msg)
{
    int rc;
    char path[DIR_MAX_PATHLEN];
    char *basepath;
    char *entry;
    ushort cluster;
    char name83[11];
    ROME_MESSAGE dir_msg;
    MSDOS_DIR_ENTRY *dir_e;
    MSDOS_DIR_ENTRY *local_e;

    if (sizeof(path) <= strlen(full_path))
    {
	rome_kprintf("dir_create_entry(): Path too long %s\n", full_path);
	return ENAMETOOLONG;
    }
    strcpy(path, full_path);
    dir_split_path(path, &basepath, &entry);

    if (0 != dir_make83(name83, entry))
    {
	return EINVAL;
    }

    /* we have to give the root dir a special treatment...
     * it's not very nice, though
     */

    if ('\0' == *basepath)
    {
	/* root dir */
	rome_kprintf("Create in root\n");

	cluster = 0;
    }
    else
    {
	/* check for the base path */
	if (0 != (rc = dir_find_path(fp, inf, basepath, &dir_e, &dir_msg)))
	{
	    rome_kprintf("dir_create_entry(): basepath %s does not exist\n",
			 basepath);
	    return rc;
	}
	cluster = dir_e->start;
	rome_retmblk(&dir_msg);
    }

    /* base path exists, check, if entry already exists */
    if (0 == dir_find_entry(fp, inf, cluster, entry, &local_e, msg))
    {
	rome_kprintf("dir_create_entry(): entry %s already exists in %s\n",
		     entry, basepath);
	return EEXIST;
    }

    /* so we can look for a free entry */
    if (0 != (rc = dir_find_free_entry(fp, inf, cluster, &local_e, msg)))
    {
	rome_kprintf("dir_create_entry(): no free entry in %s\n", basepath);
	return rc;
    }

    /* so we found a free entry, let's set the default values */

    strncpy(local_e->name, name83, 11);
    local_e->ctime_ms = 0;
    local_e->ctime = dos_dir_mktime();
    local_e->cdate = dos_dir_mkdate();
    local_e->adate = local_e->cdate;
    local_e->unused[0] = 0;
    local_e->unused[1] = 0;
    local_e->time = local_e->ctime;
    local_e->date = local_e->cdate;
    local_e->size = 0;

    *e = local_e;
    return 0;
}

static int dir_is_empty(FILE *fp, MSDOS_FS_INFO *inf, ushort start)
{
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;
    int i;
    ushort cluster = start;
    ushort last_dir_cluster = inf->root_dir_entries * sizeof(MSDOS_DIR_ENTRY)
      / inf->blksize;

    while (1==1)
    {
	int nr_entry = inf->blksize / sizeof(MSDOS_DIR_ENTRY);

	if (start)		/* not root */
	{
	    dos_fs_read_cluster(fp, inf, cluster, &msg);
	}
	else
	{
	    dos_fs_read_dir_cluster(fp, inf, cluster, &msg);
	}

	e = (MSDOS_DIR_ENTRY *)msg.b_rptr;

	for (i = 0; nr_entry > i; i++)
	{
	    if (e->name[0] == 0)
	    {
		rome_retmblk(&msg);
		return TRUE;
	    }

	    /* deleted and non-attr files skip */
	    if (DELETED_FLAG != (uchar) e->name[0])
	    {
		rome_retmblk(&msg);
		return FALSE;
	    }
	    e++;
	}
	rome_retmblk(&msg);

	if (start)		/* if not root dir, get next cluster in chain */
	{
	    cluster = dos_fat_lookup_entry(cluster, inf);
	    if (1 == cluster)
	    {
		return TRUE;
	    }
	}
	else
	{
	    if (cluster++ == last_dir_cluster)
	    {
		return TRUE;
	    }
	}
    }				/* while (1) */

    return TRUE;
}

/************************************************************************
 *
 * EXTERNAL INTERFACE FUNCTIONS
 *
 ************************************************************************/

ushort dos_dir_mktime(void)
{
    return (nvram_hour() << 11) | (nvram_minute() << 5) | (nvram_second() >> 1);
}

ushort dos_dir_mkdate(void)
{
    return ((nvram_year() - 80) << 9) | (nvram_month() << 5) | nvram_day();
}

int dos_dir_entry_exists(FILE * fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;

    if (0 != (rc = dir_find_path(fp, inf, full_path, &e, &msg)))
    {
	return rc;
    }
    rome_retmblk(&msg);

    return 0;
}

int dos_dir_dir_entry_exists(FILE *fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;

    if (0 != (rc = dir_find_path(fp, inf, full_path, &e, &msg)))
    {
	return rc;
    }

    /* check for dir */

    if (!(e->attr & ATTR_DIR))
    {
	rome_retmblk(&msg);
	return ENOTDIR;
    }

    rome_retmblk(&msg);
    return 0;
}

int dos_dir_get_entry(FILE *fp, MSDOS_FS_INFO *inf,
		      const char *full_path, MSDOS_DIR_ENTRY *entry)
{
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;
    int rc;

    if (0 != (rc = dir_find_path(fp, inf, full_path, &e, &msg)))
    {
	return rc;
    }

    /* caller gets a copy of the entry */
    *entry = *e;
    rome_retmblk(&msg);

    return 0;
}

int dos_dir_write_entry(FILE *fp, MSDOS_FS_INFO *inf,
			const char *full_path, MSDOS_DIR_ENTRY *entry)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;

    /* find the entry */

    if (0 != (rc = dir_find_path(fp, inf, full_path, &e, &msg)))
    {
	return rc;
    }
    *e = *entry;

    /* write the block */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);
    return 0;
}

int dos_dir_rename_entry(FILE *fp, MSDOS_FS_INFO *inf,
			 const char *full_path, const char *new_name)
{
    int rc;
    char name83[11];
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *e;

    if (!fp || !inf || !full_path || !new_name)
    {
	rome_kprintf("NULL pointer passed to dir_rename_entry()\n");
	return -1;
    }

    /* check, if the new name is OK */

    if (0 != dir_make83(name83, new_name))
    {
	return EINVAL;
    }

    /* find the entry */

    if (0 != (rc = dir_find_path(fp, inf, full_path, &e, &msg)))
    {
	return rc;
    }

    /* set the new name */

    strncpy(e->name, name83, 11);

    /* write the block */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);
    return 0;
}

int dos_dir_mkfile(FILE *fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *entry;
    ushort fat_entry;

    if (0 == (fat_entry = dos_fat_lookup_free(inf)))
    {
	rome_kprintf("dir_mkfile(): Could not get FAT entry for %s\n",
		     full_path);
	return ENOSPC;
    }

    /* mark the FAT entry */

    if (0 != dos_fat_write_entry(inf, fat_entry, FAT_EOF))
    {
	rome_kprintf("dir_mkfile(): Could not write FAT entry for %s\n",
		     full_path);
	return EIO;
    }

    /* try to create the entry */

    if (0 != (rc = dir_create_entry(fp, inf, full_path, &entry, &msg)))
    {
	rome_kprintf("dir_mkfile(): Could not create entry for %s\n",
		     full_path);

	/* free the FAT entry again */

	if (0 != dos_fat_write_entry(inf, fat_entry, 0))
	{
	    rome_kprintf("dir_mkfile(): Could free FAT entry for %s\n",
			 full_path);
	    return EIO;
	}
	return rc;
    }

    /* set the file attributes */

    entry->attr = ATTR_ARCH;
    entry->start = fat_entry;

    /* write the cluster */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);

    return 0;
}

int dos_dir_mkdir(FILE *fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE dir_msg;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *entry;
    ushort fat_entry;

    if (0 == (fat_entry = dos_fat_lookup_free(inf)))
    {
	rome_kprintf("dir_mkdir(): Could not get FAT entry for %s\n",
		     full_path);
	return ENOSPC;
    }
    if (0 != dos_fat_write_entry(inf, fat_entry, FAT_EOF))
    {
	rome_kprintf("dir_mkdir(): Could not write FAT entry for %s\n",
		     full_path);
	return EIO;
    }

    /* try to create the entry */

    if (0 != (rc = dir_create_entry(fp, inf, full_path, &entry, &msg)))
    {
	rome_kprintf("dir_mkdir(): Could not create entry for %s\n", full_path);

	/* free the FAT entry again */

	if (0 != dos_fat_write_entry(inf, fat_entry, 0))
	{
	    rome_kprintf("dir_mkdir(): Could not free FAT entry for %s\n",
			 full_path);
	    return EIO;
	}

	return ENOSPC;
    }

    /* set the file attributes */

    entry->attr = ATTR_DIR;
    entry->start = fat_entry;

    /* clear the cluster for the new subdir */

    dos_fs_read_cluster(fp, inf, fat_entry, &dir_msg);
    memset(dir_msg.b_rptr, 0, inf->blksize);
    dos_fs_write_cluster(fp, inf, fat_entry, &dir_msg);

    /* write the block with the new dir entry */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);

    return 0;
}

int dos_dir_rmfile(FILE *fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *entry;

    /* look for the path */

    if (0 != (rc = dir_find_path(fp, inf, full_path, &entry, &msg)))
    {
	rome_kprintf("dir_rmfile(): file %s does not exist\n", full_path);
	return rc;
    }

    /* check. if it is a file */

    if (entry->attr & ATTR_DIR)
    {
	rome_kprintf("dir_rmfile(): %s is a directory\n", full_path);
	rome_retmblk(&msg);
	return EISDIR;
    }

    /* OK, it's a file, delete the FAT chain */

    if (0 != dos_fat_free_chain(inf, entry->start))
    {
	rome_kprintf("dir_rmfile(): Could not free FAT chain for %s\n",
		     full_path);
	rome_retmblk(&msg);
	return EIO;
    }

    /* FAT is freed, mark file deleted */

    entry->name[0] = DELETED_FLAG;

    /* write the cluster */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);

    return 0;
}

int dos_dir_rmdir(FILE *fp, MSDOS_FS_INFO *inf, const char *full_path)
{
    int rc;
    ROME_MESSAGE msg;
    MSDOS_DIR_ENTRY *entry;

    /* look for the path */

    if (0 != (rc = dir_find_path(fp, inf, full_path, &entry, &msg)))
    {
	rome_kprintf("dir_rmdir(): file %s does not exist\n", full_path);
	return rc;
    }

    /* check. if it is a file */

    if (!(entry->attr & ATTR_DIR))
    {
	rome_kprintf("dir_rmdir(): %s is not a directory\n", full_path);
	rome_retmblk(&msg);
	return ENOTDIR;
    }

    /* check, if the directory is empty */

    if (!dir_is_empty(fp, inf, entry->start))
    {
	rome_kprintf("dir_rmdir(): %s is not empty\n", full_path);
	rome_retmblk(&msg);
	return ENOTEMPTY;
    }

    /* OK, it's a dir, it's empty, and it's already late...
     * delete the FAT chain
     */

    if (0 != dos_fat_free_chain(inf, entry->start))
    {
	rome_kprintf("dir_rmdir(): Could not free FAT chain for %s\n",
		     full_path);
	rome_retmblk(&msg);
	return EIO;
    }

    /* FAT is freed, mark file deleted */

    entry->name[0] = DELETED_FLAG;

    /* write the cluster */

    rome_putmblk(fp, &msg);
    rome_await_message(&msg, 0);

    return 0;
}

int dos_dir_read_dir(DOS_OPENFILE *of, FSLIB_DIRENT *dir, int count, int *read)
{
    int i;
    ROME_MESSAGE msg;
    ushort cluster;
    ushort start = of->start;
    FILE *fp = of->dev->fp;
    MSDOS_FS_INFO *inf = &of->dev->inf;
    FSLIB_DIRENT *entry = dir;
    ushort last_dir_cluster = inf->root_dir_entries * sizeof(MSDOS_DIR_ENTRY)
      / inf->blksize;

    *read = 0;

    /* as the root dir and any other subdir have to be treated in a different
     * way, we always have to check and do different things for root- and
     * subdirs. This is because of the MSDOS DIR layout... (Greetings to Bill)
     *
     * IT'S NOT OUR FAULT!
     */

    /* check. if it is a file */

    if (FSLIB_TYPE_DIR != of->type)
    {
	rome_kprintf("dir_read_dir(): %s is not a directory\n", of->filename);
	return ENOTDIR;
    }

    /* check, if the directory is empty */

    if (dir_is_empty(fp, inf, start))
    {
	return -1;
    }

    /* start to create the fslib_dirent structure */

    cluster = of->cluster;

    while (1==1)
    {
	MSDOS_DIR_ENTRY *e;
	int nr_entry = inf->blksize / sizeof(MSDOS_DIR_ENTRY);

	if (start)		/* root dir? */
	{
	    /* no, read normal cluster from filespace */
	    dos_fs_read_cluster(fp, inf, cluster, &msg);
	}
	else
	{
	    /* yes, read cluster from rootdir space */
	    dos_fs_read_dir_cluster(fp, inf, cluster, &msg);
	}

	e = (MSDOS_DIR_ENTRY *)msg.b_rptr;

	if (of->fpos >= nr_entry)
	{
	    rome_kprintf("dir_read_dir(): INTERNAL ERROR fpos %d, "
			 "resetting fpos to 0...\n", of->fpos);
	    of->fpos = 0;
	}
	e += of->fpos;

	for (i = of->fpos; nr_entry > i; i++, e++)
	{
	    uchar *f = e->name;
            char *t = entry->name;
	    int c = 8;

	    /* check if last entry (end of dir) */

	    if (0 == *f)
	    {
		rome_retmblk(&msg);
		return 0;
	    }

	    /* check for deleted entry or 'invalid (WIN95)' entry
	     * or '.' and '..' entries generated by LINUX
	     */

	    if (DELETED_FLAG == *f || 0x0f == e->attr || '.' == *f)
	    {
		of->fpos++;
		continue;
	    }

	    /* OK, this should be a valid entry,
	     * check, if we have place for it
	     */

	    if (!count)
	    {
		/* no more place in user buffer: return. */
		rome_retmblk(&msg);
		return 0;
	    }

	    while (*f != ' ' && c--)
	    {
		*t++ = *f++;
	    }

	    f = e->ext;
	    if  (*f != ' ')
	    {
		*t++ = '.';
		c = 3;
		while (*f && c--)
		{
		    *t++ = *f++;
		}
	    }
	    *t = '\0';

	    /* set the attributes */

	    if (e->attr & ATTR_DIR)
	    {
		entry->attr.type = FSLIB_TYPE_DIR;
	    }
	    else
	    {
		entry->attr.type = FSLIB_TYPE_FILE;
	    }

	    entry->attr.size = e->size;

	    entry->attr.ctime[0] = 80+(e->cdate >> 9);
	    entry->attr.ctime[1] = (e->cdate >> 5) & 15;
	    entry->attr.ctime[2] = (e->cdate & 31);
	    entry->attr.ctime[3] = (e->ctime >> 11);
	    entry->attr.ctime[4] = (e->ctime >> 5) & 63;
	    entry->attr.ctime[5] = (e->ctime & 31) << 1;
	    entry->attr.ctime[6] = e->ctime_ms / 256;
	    entry->attr.ctime[7] = e->ctime_ms & 255;

	    entry->attr.atime[0] = 80+(e->adate >> 9);
	    entry->attr.atime[1] = (e->adate >> 5) & 15;
	    entry->attr.atime[2] = (e->adate & 31);
	    entry->attr.atime[3] = 0;
	    entry->attr.atime[4] = 0;
	    entry->attr.atime[5] = 0;
	    entry->attr.atime[6] = 0;
	    entry->attr.atime[7] = 0;

	    entry->attr.mtime[0] = 80+(e->date >> 9);
	    entry->attr.mtime[1] = (e->date >> 5) & 15;
	    entry->attr.mtime[2] = (e->date & 31);
	    entry->attr.mtime[3] = (e->time >> 11);
	    entry->attr.mtime[4] = (e->time >> 5) & 63;
	    entry->attr.mtime[5] = (e->time & 31) << 1;
	    entry->attr.mtime[6] = 0;
	    entry->attr.mtime[7] = 0;

	    /* next entry */

	    of->fpos++;
	    entry++;
	    count--;
	    (*read)++;
	}			/* for */

	/* give the current mblk back */

	rome_retmblk(&msg);

	if (start)		/* if not root dir, get next cluster in chain */
	{
	    cluster = dos_fat_lookup_entry(cluster, inf);
	    if (1 == cluster)
	    {
		return 0;
	    }
	}
	else
	{
	    /* root dir entry, check for last cluster */
	    if (cluster++ == last_dir_cluster)
	    {
		return 0;
	    }
	}
	/* remember current position in dir */
	of->cluster = cluster;
	of->fpos = 0;
    }				/* while 1 */

    return 0;
}
