/* **************************************************************************
 *    Copyright (C) 2005 by Rob Brown                                       *
 *    rob@cobbleware.com                                                    *
 *                                                                          *
 *    C Implementation: $MODULE$                                            *
 *                                                                          *
 *    This program is free software; you can redistribute it and/or modify  *
 *    it under the terms of the GNU General Public License as published by  *
 *    the Free Software Foundation; either version 2 of the License, or     *
 *    (at your option) any later version.                                   *
 *                                                                          *
 *    This program is distributed in the hope that it will be useful,       *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *    GNU General Public License for more details.                          *
 *                                                                          *
 *    You should have received a copy of the GNU General Public License     *
 *    along with this program; if not, write to the                         *
 *    Free Software Foundation, Inc.,                                       *
 *    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 * **************************************************************************/

#include "sdk2x/mmsp2.h"
#include "sdk2x/sdk2x_priv.h"

static const sdk2x_RegisterTable_t LcdRegisters[] = {
	{ nMSP_DISPCSETREG, 0x5900, 2 },

	{ nMSP_DPC_INTR, 0x0000, 2 },					/* mode setting, rgb666, interlace none */

	{ nMSP_DPC_CLKCNTL, 0x0010, 2 },				/* clock source */

	{ nMSP_DPC_X_MAX, 319, 2 },					/* set frame size */
	{ nMSP_DPC_Y_MAX, 239, 2 },					/* */

	{ nMSP_DPC_HS_WIDTH, 0x041d, 2 },			/* DPC_SetHSyncParm ( HorSyncWidth, HorFrontPorch, HorBackPorch, HorPolarity) */
	{ nMSP_DPC_HS_END, 0x0009, 2 },				/* */
	{ nMSP_DPC_HS_STR, 0x0009, 2 },				/* */
	{ nMSP_DPC_DE, 0x0250, 2 },					/* */

	{ nMSP_DPC_FPIPOL1, 0x10, 2 },				/* */

	{ nMSP_DPC_V_SYNC, 0x0f03, 2 },				/* DPC_SetVSyncParm ( VerSyncWidth, VerFrontPorch, VerBackPorch, VerPolarity) */
	{ nMSP_DPC_V_END, 0x0816, 2 },				/* */

	/* Video Post Processor MLC setup */
	{ nMSP_MLC_OVLAY_CNTR, 0x1004, 2 },			/* Uses RGB layer region 1 only */

	{ nMSP_MLC_STL_CNTL, 0x4AB, 2 },				/* */
	{ nMSP_MLC_STL_MIXMUX, 0, 2 },				/* */

	{ nMSP_MLC_STL_ALPHAL, 0x0008, 2 },			/* */
	{ nMSP_MLC_STL_ALPHAH, 0x0000, 2 },			/* */

	{ nMSP_MLC_STL1_STX, 0, 2 },					/* */
	{ nMSP_MLC_STL1_ENDX, 319, 2 },				/* */
	{ nMSP_MLC_STL1_STY, 0, 2 },					/* */
	{ nMSP_MLC_STL1_ENDY, 239, 2 },				/* */

	{ nMSP_MLC_STL_CKEY_GB, 0xffff, 2 },		/* */
	{ nMSP_MLC_STL_CKEY_R, 0x00ff, 2 },			/* */

	{ nMSP_MLC_STL_HSC, 1024, 2 },				/* 1024 = scale factor 1.00 */
	{ nMSP_MLC_STL_VSCL, 640, 2 },				/* Twice Hwidth */
	{ nMSP_MLC_STL_VSCH, 0, 2 },					/* */
	{ nMSP_MLC_STL_HW, 640, 2 },					/* Twice Hwidth */

	{ nMSP_GPIOAALTFNLOW, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOBALTFNLOW, 0xAA50, 2 },			/* */
	{ nMSP_GPIOCALTFNLOW, 0x0000, 2 },			/* */
	{ nMSP_GPIODALTFNLOW, 0x0AAA, 2 },			/* */
	{ nMSP_GPIOEALTFNLOW, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOFALTFNLOW, 0xAAAA, 2 },			/* 0x02AA OK */
	{ nMSP_GPIOGALTFNLOW, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOHALTFNLOW, 0x0955, 2 },			/* 0x0055 OK */
	{ nMSP_GPIOIALTFNLOW, 0xAA0A, 2 },			/* */
	{ nMSP_GPIOJALTFNLOW, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOKALTFNLOW, 0x0000, 2 },			/* */
	{ nMSP_GPIOLALTFNLOW, 0xAAAA, 2 },			/* 0x2AAA OK */
	{ nMSP_GPIOMALTFNLOW, 0x0000, 2 },			/* */
	{ nMSP_GPIONALTFNLOW, 0x0000, 2 },			/* */
	{ nMSP_GPIOOALTFNLOW, 0x0000, 2 },			/* */
	{ nMSP_GPIOAALTFNHI, 0x000A, 2 },			/* */
	{ nMSP_GPIOBALTFNHI, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOCALTFNHI, 0x0000, 2 },			/* */
	{ nMSP_GPIODALTFNHI, 0x002A, 2 },			/* 0x00A8 OK */
	{ nMSP_GPIOEALTFNHI, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOFALTFNHI, 0x000A, 2 },			/* 0x02AA OK */
	{ nMSP_GPIOGALTFNHI, 0xAAAA, 2 },			/* */
	{ nMSP_GPIOHALTFNHI, 0x0000, 2 },			/* 0x0012 OK */
	{ nMSP_GPIOIALTFNHI, 0x8AAA, 2 },			/* */
	{ nMSP_GPIOJALTFNHI, 0x16AA, 2 },			/* */
	{ nMSP_GPIOKALTFNHI, 0x0000, 2 },			/* */
	{ nMSP_GPIOLALTFNHI, 0x002A, 2 },			/* 0x0055 OK */
	{ nMSP_GPIOMALTFNHI, 0x0000, 2 },			/* */
	{ nMSP_GPIONALTFNHI, 0x0000, 2 },			/* */
	{ nMSP_GPIOOALTFNHI, 0x0000, 2 },			/* */
	{ nMSP_SYSCLKENREG, 0xFFFF, 2 },				/* */
	{ nMSP_VGCLKENREG, 0xFFFF, 2 },				/* */
	{ nMSP_SYSCSETREG, 0x0000, 2 },				/* */
	{ nMSP_ESYSCSETREG, 0x4170, 2 },				/* */
	{ nMSP_DISPCSETREG, 0x4f00, 2 },				/* */
	{ nMSP_IMAGCSETREG, 0x5e00, 2 },				/* */
	{ nMSP_DPC_FPIPOL2, 0xFFFF, 2 },				/* */
	{ nMSP_DPC_FPIPOL3, 0x00FF, 2 },				/* */
	{ nMSP_GPIOAPUENB, 0xFFFF, 2 },				/* */
	{ nMSP_GPIOBPUENB, 0xFFFF, 2 },				/* */
	{ nMSP_GPIOCPUENB, 0xFFFF, 2 },				/* */
	{ nMSP_GPIODPUENB, 0x3FFC, 2 },				/* */
	{ nMSP_GPIOEPUENB, 0x0000, 2 },				/* */
	{ nMSP_GPIOFPUENB, 0x0000, 2 },				/* */
	{ nMSP_GPIOGPUENB, 0x0000, 2 },				/* */
	{ nMSP_GPIOHPUENB, 0x007E, 2 },				/* */
	{ nMSP_GPIOIPUENB, 0x4200, 2 },				/* */
	{ nMSP_GPIOJPUENB, 0x0000, 2 },				/* */
	{ nMSP_GPIOKPUENB, 0x00FF, 2 },				/* */
	{ nMSP_GPIOLPUENB, 0x7FFF, 2 },				/* */
	{ nMSP_GPIOMPUENB, 0x01FF, 2 },				/* */
	{ nMSP_GPIONPUENB, 0x00EF, 2 },				/* */
	{ nMSP_GPIOOPUENB, 0x003F, 2 },				/* */
#if 0
	{ nMSP_PPMCTRL, 0x4000, 2 },					/* */
	{ nMSP_MPEGIFCONT, 0x0003, 2 },				/* */
	{ nMSP_DECSSCTRL, 0x0008, 2 },				/* */
	{ nMSP_DECSSKEY0, 0xEBF1, 2 },				/* */
	{ nMSP_DECSSKEY1, 0xC080, 2 },				/* */
	{ nMSP_DECSSKEY2, 0x7C31, 2 },				/* */
	{ nMSP_DECSSDATA, 0x5020, 2 },				/* */
	{ nMSP_DECSSNUM, 0x0000, 2 },					/* */
	{ nMSP_DECSSCTRL, 0x0000, 2 },				/* */
	{ nMSP_SC_CMD, 0x0100, 2 },					/* */
	{ nMSP_SC_STATUS, 0x0004, 2 },				/* */
	{ nMSP_SC_MIRROR, 0x0000, 2 },				/* */
	{ nMSP_SC_PRE_VRATIO, 0x0400, 2 },			/* */
	{ nMSP_SC_PRE_HRATIO, 0x0400, 2 },			/* */
	{ nMSP_SC_POST_VRATIOL, 0x4000, 2 },		/* */
	{ nMSP_SC_POST_VRATIOH, 0x0000, 2 },		/* */
	{ nMSP_SC_POST_HRATIOL, 0x4000, 2 },		/* */
	{ nMSP_SC_POST_HRATIOH, 0x0000, 2 },		/* */
	{ nMSP_SC_SRC_ODD_ADDRL, 0x0000, 2 },		/* */
	{ nMSP_SC_SRC_ODD_ADDRH, 0x0000, 2 },		/* */
	{ nMSP_SC_SRC_EVEN_ADDRL, 0x0000, 2 },		/* */
	{ nMSP_SC_SRC_EVEN_ADDRH, 0x0000, 2 },		/* */
	{ nMSP_SC_DST_ADDRL, 0x0000, 2 },			/* */
	{ nMSP_SC_DST_ADDRH, 0x0000, 2 },			/* */
	{ nMSP_SC_SRC_PXL_HEIGHT, 0x00F0, 2 },		/* */
	{ nMSP_SC_SRC_PXL_WIDTH, 0x0140, 2 },		/* */
	{ nMSP_SC_SRC_PXL_REQCNT, 0x0140, 2 },		/* */
	{ nMSP_SC_DST_PXL_HEIGHT, 0x00F0, 2 },		/* */
	{ nMSP_SC_DST_PXL_WIDTH, 0x0140, 2 },		/* */
	{ nMSP_SC_DST_WPXL_WIDTH, 0x0000, 2 },		/* */
	{ nMSP_SC_LUMA_OFFSET, 0x0000, 2 },			/* */
	{ nMSP_SC_CB_OFFSET, 0x0000, 2 },			/* */
	{ nMSP_SC_CR_OFFSET, 0x0000, 2 },			/* */
	{ nMSP_SC_DELAY, 0x0073, 2 },					/* */
	{ nMSP_SC_MEM_CNTR, 0x0010, 2 },				/* */
	{ nMSP_SC_IRQ, 0x0000, 2 },					/* */
	{ nMSP_ISP_CON_1, 0x0000, 2 },				/* */
	{ nMSP_ISP_CON_2, 0x0100, 2 },				/* */
	{ nMSP_ISP_INT_CON, 0x4400, 2 },				/* */
	{ nMSP_ISP_VPERIOD, 0x0106, 2 },				/* */
	{ nMSP_ISP_VPW, 0x000A, 2 },					/* */
	{ nMSP_ISP_HPERIOD, 0x0187, 2 },				/* */
	{ nMSP_ISP_HPW, 0x2032, 2 },					/* */
	{ nMSP_ISP_HACT_ST, 0x000B, 2 },				/* */
	{ nMSP_ISP_H_WIDTH, 0x014B, 2 },				/* */
	{ nMSP_ISP_VACT_ST, 0x000A, 2 },				/* */
	{ nMSP_ISP_V_HEIGHT, 0x00FA, 2 },			/* */
	{ nMSP_ISP_SRC_HWIDTH, 0x0000, 2 },			/* */
	{ nMSP_ISP_RGBSV, 0x0005, 2 },				/* */
	{ nMSP_ISP_RGBEV, 0x00FF, 2 },				/* */
	{ nMSP_ISP_RGBSH, 0x0005, 2 },				/* */
	{ nMSP_ISP_RGBEH, 0x014F, 2 },				/* */
	{ nMSP_ISP_AESV, 0x000A, 2 },					/* */
	{ nMSP_ISP_AEEV, 0x00FA, 2 },					/* */
	{ nMSP_ISP_AESH, 0x000A, 2 },					/* */
	{ nMSP_ISP_AEEH, 0x014A, 2 },					/* */
	{ nMSP_ISP_AELIM, 0xFF00, 2 },				/* */
	{ nMSP_ISP_AEVSTEP, 0x003C, 2 },				/* */
	{ nMSP_ISP_AEHSTEP, 0x0050, 2 },				/* */
	{ nMSP_ISP_AFSV, 0x000A, 2 },					/* */
	{ nMSP_ISP_AFEV, 0x00FA, 2 },					/* */
	{ nMSP_ISP_AFSH, 0x000A, 2 },					/* */
	{ nMSP_ISP_AFEH, 0x014A, 2 },					/* */
	{ nMSP_ISP_CNTRSTGAIN, 0x0003, 2 },			/* */
	{ nMSP_ISP_CNTRSTLIM, 0xFF00, 2 },			/* */
	{ nMSP_ISP_RGBREC, 0x8000, 2 },				/* */
	{ nMSP_ISP_HUECB0AB, 0x4000, 2 },			/* */
	{ nMSP_ISP_HUECR0AB, 0x0040, 2 },			/* */
	{ nMSP_ISP_HUECB1AB, 0x4000, 2 },			/* */
	{ nMSP_ISP_HUECR1AB, 0x0040, 2 },			/* */
	{ nMSP_ISP_HUECB2AB, 0x4000, 2 },			/* */
	{ nMSP_ISP_HUECR2AB, 0x0040, 2 },			/* */
	{ nMSP_ISP_HUECB3AB, 0x4000, 2 },			/* */
	{ nMSP_ISP_HUECR3AB, 0x0040, 2 },			/* */
	{ nMSP_ISP_YGAIN, 0x0013, 2 },				/* */
	{ nMSP_ISP_CDLY, 0x0002, 2 },					/* */
	{ nMSP_ISP_BRIGHT, 0x0000, 2 },				/* */
	{ nMSP_ISP_YCLIP, 0xFFFF, 2 },				/* */
	{ nMSP_ISP_YCLIPSLANT, 0x0004, 2 },			/* */
	{ nMSP_ISP_CSUPSLANT, 0x0004, 2 },			/* */
	{ nMSP_ISP_SLICELMT, 0x0008, 2 },			/* */
	{ nMSP_ISP_WBLMT, 0xFF00, 2 },				/* */
	{ nMSP_ISP_WBACCCNTH, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBACCCNTL, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCBACCH, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCBACCL, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCRACCH, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCRACCL, 0x0000, 2 },			/* */
	{ nMSP_ISP_AFACCH, 0x0000, 2 },				/* */
	{ nMSP_ISP_AFACCL, 0x0000, 2 },				/* */
	{ nMSP_ISP_WBCOORD0, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD1, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD2, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD3, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD4, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD5, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD6, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD7, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD8, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD9, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD10, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD11, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD12, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD13, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD14, 0x0000, 2 },			/* */
	{ nMSP_ISP_WBCOORD15, 0x0000, 2 },			/* */
	{ nMSP_ISP_MEMADDRL1, 0x0000, 2 },			/* */
	{ nMSP_ISP_MEMADDRH1, 0x0080, 2 },			/* */
	{ nMSP_ISP_GAMMA, 0x0000, 2 },				/* */
	{ nMSP_ISP_GAMMD, 0x0000, 2 },				/* */
	{ nMSP_ISP_AEACCH, 0x0000, 2 },				/* */
	{ nMSP_ISP_AEACCL, 0x0000, 2 },				/* */
	{ nMSP_ISP_CNTST_LLCNTL, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_LLCNTH, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_HLCNTL, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_HLCNTH, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_LLACCL, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_LLACCH, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_HLACCL, 0x0000, 2 },		/* */
	{ nMSP_ISP_CNTST_HLACCH, 0x0000, 2 },		/* */
	{ nMSP_ISP_MEMRGBADDRL, 0x0000, 2 },		/* */
	{ nMSP_ISP_MEMRGBADDRH, 0x0080, 2 },		/* */
	{ nMSP_ISP_MEMADDRL2, 0x0000, 2 },			/* */
	{ nMSP_ISP_MEMADDRH2, 0x0080, 2 },			/* */
	{ nMSP_ISP_DST_HWIDTH, 0x0000, 2 },			/* */
	{ nMSP_MLC_OVLAY_CNTR, 0x1004, 2 },			/* */
	{ nMSP_MLC_YUV_EFFECT, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUV_CNTL, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUVA_TP_HSC, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_HSC, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_HSCL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_HSCH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_HSCL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_HSCH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_PXW, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_PXW, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUVA_ENDX, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUVA_TP_STY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_ENDY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_ENDY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_OADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_OADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_EADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_TP_EADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_OADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_OADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_EADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVA_BT_EADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_HSC, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_HSC, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_VSCL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_VSCH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_VSCL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_VSCH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_PXW, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_PXW, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUVB_ENDX, 0x0000, 2 },			/* */
	{ nMSP_MLC_YUVB_TP_STY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_ENDY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_ENDY, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_OADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_OADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_EADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_TP_EADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_OADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_OADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_EADRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_YUVB_BT_EADRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_STL_CNTL, 0x04AB, 2 },			/* */
	{ nMSP_MLC_STL_MIXMUX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL_ALPHAL, 0x0008, 2 },			/* */
	{ nMSP_MLC_STL_ALPHAH, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL1_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL1_ENDX, 0x013F, 2 },			/* */
	{ nMSP_MLC_STL1_STY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL1_ENDY, 0x00EF, 2 },			/* */
	{ nMSP_MLC_STL2_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL2_ENDX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL2_STY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL2_ENDY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL3_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL3_ENDX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL3_STY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL3_ENDY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL4_STX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL4_ENDX, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL4_STY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL4_ENDY, 0x0000, 2 },			/* */
	{ nMSP_MLC_STL_CKEY_GB, 0xFFFF, 2 },		/* */
	{ nMSP_MLC_STL_CKEY_R, 0x00FF, 2 },			/* */
	{ nMSP_MLC_STL_HSC, 0x0400, 2 },				/* */
	{ nMSP_MLC_STL_VSCL, 0x0280, 2 },			/* */
	{ nMSP_MLC_STL_VSCH, 0x0280, 2 },			/* */
	{ nMSP_MLC_STL_HW, 0x0280, 2 },				/* */
	{ nMSP_MLC_STL_OADRL, 0x1000, 2 },			/* */
	{ nMSP_MLC_STL_OADRH, 0x0310, 2 },			/* */
	{ nMSP_MLC_STL_EADRL, 0x1000, 2 },			/* */
	{ nMSP_MLC_STL_EADRH, 0x0310, 2 },			/* */
	{ nMSP_MLC_HWC_CNTR, 0xFF00, 2 },			/* */
	{ nMSP_MLC_HWC_STX, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_STY, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_FGR, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_FB, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_BGR, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_BB, 0x0000, 2 },				/* */
	{ nMSP_MLC_HWC_OADRL, 0x0000, 2 },			/* */
	{ nMSP_MLC_HWC_OADRH, 0x0000, 2 },			/* */
	{ nMSP_MLC_HWC_EADRL, 0x0000, 2 },			/* */
	{ nMSP_MLC_HWC_EADRH, 0x0000, 2 },			/* */
	{ nMSP_MLC_LUMA_ENH, 0x0000, 2 },			/* */
	{ nMSP_MLC_HUECB1AB, 0x4000, 2 },			/* */
	{ nMSP_MLC_HUECR1AB, 0x0040, 2 },			/* */
	{ nMSP_MLC_HUECB2AB, 0x4000, 2 },			/* */
	{ nMSP_MLC_HUECR2AB, 0x0040, 2 },			/* */
	{ nMSP_MLC_HUECB3AB, 0x4000, 2 },			/* */
	{ nMSP_MLC_HUECR3AB, 0x0040, 2 },			/* */
	{ nMSP_MLC_HUECB4AB, 0x4000, 2 },			/* */
	{ nMSP_MLC_HUECR4AB, 0x0040, 2 },			/* */
	{ nMSP_MLC_DITHER, 0x0000, 2 },				/* */
	{ nMSP_MLC_OSD_PALLT_A, 0x0000, 2 },		/* */
	{ nMSP_MLC_STL_PALLT_A, 0x0000, 2 },		/* */
	{ nMSP_MLC_GAMMA_A, 0x0000, 2 },				/* */
	{ nMSP_MLC_SPU_CTRL, 0x0000, 2 },			/* */
	{ nMSP_MLC_SPU_DELAY, 0x0000, 2 },			/* */
	{ nMSP_MLC_SPU_BASEI_ADDRL, 0x0000, 2 },	/* */
	{ nMSP_MLC_SPU_BASEI_ADDRH, 0x0000, 2 },	/* */
	{ nMSP_MLC_SPU_HLI_ADDRL, 0x0000, 2 },		/* */
	{ nMSP_MLC_SPU_HLI_ADDRH, 0x0000, 2 },		/* */
	{ nMSP_MLC_SPU_PAL, 0xF7FF, 2 },				/* */
	{ nMSP_DPC_CLKCNTL, 0x0010, 2 },				/* */
	{ nMSP_DPC_FPICNTL, 0x0000, 2 },				/* */
	{ nMSP_DPC_FPIPOL1, 0x0003, 2 },				/* */
	{ nMSP_DPC_FPIPOL2, 0xFFFF, 2 },				/* */
	{ nMSP_DPC_FPIPOL3, 0x00FF, 2 },				/* */
	{ nMSP_DPC_FPIATV1, 0xFFFF, 2 },				/* */
	{ nMSP_DPC_FPIATV2, 0xFFFF, 2 },				/* */
	{ nMSP_DPC_FPIATV3, 0x00FF, 2 },				/* */
	{ nMSP_DPC_X_MAX, 0x013F, 2 },				/* */
	{ nMSP_DPC_Y_MAX, 0x00EF, 2 },				/* */
	{ nMSP_DPC_HS_WIDTH, 0x041D, 2 },			/* */
	{ nMSP_DPC_HS_STR, 0x0009, 2 },				/* */
	{ nMSP_DPC_HS_END, 0x0009, 2 },				/* */
	{ nMSP_DPC_V_SYNC, 0x0F03, 2 },				/* */
	{ nMSP_DPC_V_END, 0x0816, 2 },				/* */
	{ nMSP_DPC_TV_BNK, 0x0000, 2 },				/* */
	{ nMSP_DPC_DE, 0x0250, 2 },					/* */
	{ nMSP_DPC_PS, 0x0000, 2 },					/* */
	{ nMSP_DPC_FG, 0x0000, 2 },					/* */
	{ nMSP_DPC_LP, 0x0000, 2 },					/* */
	{ nMSP_DPC_CLKVH, 0x0000, 2 },				/* */
	{ nMSP_DPC_CLKVL, 0x0000, 2 },				/* */
	{ nMSP_DPC_POL, 0x0000, 2 },					/* */
	{ nMSP_DPC_CISSYNC, 0x0000, 2 },				/* */
	{ nMSP_DPC_MID_SYNC, 0x0000, 2 },			/* */
	{ nMSP_DPC_C_SYNC, 0x0064, 2 },				/* */
	{ nMSP_DPC_Y_BLANK, 0x00FC, 2 },				/* */
	{ nMSP_DPC_C_BLANK, 0x0200, 2 },				/* */
	{ nMSP_DPC_YP_CSYNC, 0x02EC, 2 },			/* */
	{ nMSP_DPC_YN_CSYNC, 0x02EC, 2 },			/* */
	{ nMSP_DPC_CP_CSYNC, 0x0010, 2 },			/* */
	{ nMSP_DPC_CN_CSYNC, 0x0114, 2 },			/* */
	{ nMSP_DPC_INTR, 0x0000, 2 },					/* */
	{ nMSP_DPC_CNTL, 0x0005, 2 },					/* */
#endif
	{ 0, 0, 0 }											/* Terminating entry */
};

void sdk2x_LcdDisable (void)
{
	MSP_DPC_FPIATV1 = 0;
	MSP_DPC_FPIATV2 = 0;
	MSP_DPC_FPIATV3 = 0;
	MSP_DPC_CNTL   = 4;
	MSP_DPC_FPICNTL = 1;
}

void sdk2x_LcdEnable (void)
{
	int i;

	MSP_GPIOBOUT &= ~0x0c;
	MSP_GPIODOUT &= ~0x10;
	MSP_GPIOHOUT &= ~0x06;
	MSP_GPIOHOUT |= 0x08;

	MSP_DPC_FPIATV1 = 0xffff;
	MSP_DPC_FPIATV2 = 0xffff;
	MSP_DPC_FPIATV3 = 0xffff;
	MSP_DPC_FPICNTL = 0;
	MSP_DPC_CNTL    |= 0x0001;

	for (i=0;i<1000000;i++);

	MSP_GPIOBOUT |= 0x0c;
	MSP_GPIODOUT |= 0x10;
	MSP_GPIOHOUT &= ~0x08;
	MSP_GPIOHOUT |= 0x06;
}

void sdk2x_LcdInit (unsigned int nBufferAddress)
{
	sdk2x_LcdDisable();

	sdk2x_SysInitTable (LcdRegisters);

	sdk2x_LcdSetFrameBuffer (nBufferAddress);

	sdk2x_LcdEnable();
}

void sdk2x_LcdSetFrameBuffer (unsigned int nBufferAddress)
{
	MSP_MLC_STL_OADRL = (unsigned short)(nBufferAddress & 0xffff);
	MSP_MLC_STL_OADRH = (unsigned short)(nBufferAddress >> 16);
	MSP_MLC_STL_EADRL = (unsigned short)(nBufferAddress & 0xffff);
	MSP_MLC_STL_EADRH = (unsigned short)(nBufferAddress >> 16);
}
